/*
   Grr RSS Reader
   
   Copyright (C) 2006, 2007 Guenther Noack <guenther@unix-ag.uni-kl.de>
   Copyright (C) 2009  GNUstep Application Team
                       Riccardo Mottola

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 3 of the License, or (at your option) any later version.
 
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
 
   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA. 
*/

#import "Components.h"

NSString* ComponentDidUpdateNotification = @"ComponentDidUpdateNotification";

@implementation ViewProvidingComponent

-(id) init
{
  if ((self = [super init]) != nil)
    {
      BOOL nibLoaded;
      NSString *nibName;

      nibName = NSStringFromClass([self class]);
      nibLoaded = [NSBundle loadNibNamed:nibName owner:self];
      if (nibLoaded == NO)
	{
	  NSLog(@"ViewProvidingComponent: Failed to load nib %@.", nibName);
	  return nil;
	}
      [_view retain]; // make sure _view is ours!
    }
    
  return self;
}

-(void)dealloc
{
    [_view release];
    [super dealloc];
}

-(NSView*) view
{
    return [[_view retain] autorelease];
}

-(void) notifyChanges
{
    [[NSNotificationCenter defaultCenter] postNotificationName: ComponentDidUpdateNotification
                                                        object: self];
}

@end
